/** \file start.cpp
Testbeispiel (ATMEGA 328p 16Mhz)
===================================================
\author: Schnegger 2016 4AHEL
\date: 29.01.16

Garagen Steuerung: PD2(INT0) Input :Start Taste bzw. fallende Flanke
PD3(INT1) Input :Ready sagt, dass Tre offen ist ==> abgelaufene Zeit abspeichern
PC0,PC1 Motorsignale: Motor-left, Motor-right
ADC4(PC4): Analoge Spannung fr ffnungszeit-Einstellung zwischen 0 und 100s
*/

#include <avr/io.h>
#include <avr/interrupt.h>

#define  MOTORLEFT 0
#define  MOTORRIGHT 1

volatile int savedOpeningTime=-9; //no time saved is marked by a negative value
bool openingPhase = 0; //check if open or close
uint16_t openingTime=0;
uint16_t poti=0;
uint16_t ms=0;  //1000 milliseconds max
uint8_t time=0;  //100 seconds max
void setup();

int main(void)
{
	setup();
	while(1){
	}
}
void setup()
{
	
	//set Port,DDR
	DDRD=0; //whole PortD input dont need pullups
	DDRC|=_BV(MOTORLEFT)|_BV(MOTORRIGHT);
	PORTC=0;  // Stop the Motor
	
	
	//INT0 falling edge
	EICRA|=_BV(ISC01); //falling edge
	EIMSK|=_BV(INT0); //enable interrupt
	
	//INT1 rising edge
	EICRA|=_BV(ISC11)|_BV(ISC10); //rising edge
	EIMSK|=_BV(INT1); //enable interrupt

	//ADU configuration
	ADCSRA|=_BV(ADEN); //enable ADC
	ADMUX|=_BV(REFS0); ADMUX&=~_BV(REFS1); //external Reference Voltage
	ADMUX|=_BV(MUX2); //ADC4(PC4)
	
	//Timer0
	//enable Overflow every 1ms    62.5ns*64*256 = 1ms ==> Prescaler 64 CS0=3
	// TCCR0B|=_BV(CS00)|_BV(CS01);  dont start yet
	TCNT0=0; //set 0
	TIMSK0|=_BV(TOIE0); //Enable overflow interrupt
	
	sei();
}


ISR(TIMER0_OVF_vect) //every 1ms
{
	ms++;
	if(ms>=1000)
	{   ms=0;
		time++; //one second more
	}

	if ((time>=100)||(time>=openingTime)){
		// we are ready, gate is open or closed
		// stop the timer when openingTime is over or ready signal received (INT1)
		//stop the timer
		TCCR0B&=~(_BV(CS00)|_BV(CS01));
		//stop the motor
		PORTC=0; 
	}
}


//ISR start
ISR(INT0_vect)
{   if(savedOpeningTime==-9)
	{
		//read analog value to get opening time within 0 and 100s   (ADU values 0 .. 1024)
		ADCSRA|=_BV(ADSC); //start of conversion
		while ( ADCSRA & _BV(ADSC) );  // wait for conversion finished
		poti = ADCW;
		openingTime = poti/10; //seconds to wait for opening
	}
	else
	{
		openingTime = savedOpeningTime;
	}

	
	if (openingPhase)
	{
		PORTC|=_BV(MOTORRIGHT); //open
		PORTC&=~_BV(MOTORLEFT);
	}
	else
	{
		PORTC|=_BV(MOTORLEFT); //close
		PORTC&=~_BV(MOTORRIGHT);
	}
	time=0; // 100 s max
	//reset and start the timer
	TCNT0=0;
	TCCR0B|=_BV(CS00)|_BV(CS01);

	openingPhase = !openingPhase; //next time we close the door
}

//ISR INT1 ready signal
ISR(INT1_vect)
{
	//stop the timer
	TCCR0B&=~(_BV(CS00)|_BV(CS01));

	// save new opening time
	savedOpeningTime = time;
}



